cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c       Programmer: Eugene Tsiang                                      c
c       AXAF Mission Support Team                                      c
c       Date: 8/26/96                                                  c
c       Place: Smithsonian Astrophysical Observatory, Cambridge, MA    c
c       Spectral Analysis Program for XSPEC Revision 6.00              c
c       Module name:  jmmgf1.f
c       Modifications:
c       9/4/96 tail parameters changed using logs to prevent instability
c	10/31/96 previous tail parameters implementation corrected
c       11/4/96 NaN in xspec caused by tail parameters overflowing
c            cured by taking inverse and causing underflowing
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      double complex function jmmgf(slap) 
c moment generating function for a line
      include 'params.for'
      external initje

      double complex slap, sglmgf, sglshelf, plymgf
      external plymgf
      if (shelfswitch.eq.0) then
          jmmgf = sglmgf(slap,plymgf)*plymgf(slap)
      else
          jmmgf = sglshelf(slap,plymgf)*plymgf(slap)              
*      jmmgf =  sglmgf(slap,plymgf)
      endif     
      return
      end

      double complex function dgjmgf(slap)
c moment generating function for a line
      include 'params.for'
      external initje
c make sure this is changed to 'params.f' in xspec
*      include 'params.f'

      double complex slap, sglmgf, sglshelf, dgmgf
      external dgmgf
      if (shelfswitch.eq.0) then      
          dgjmgf = sglmgf(slap, dgmgf)*dgmgf(slap)
      else
          dgjmgf = sglshelf(slap, dgmgf)*dgmgf(slap)    
*      dgjmgf = sglmgf(slap, dgmgf)                              
      endif
      return
      end
            
      double complex function binmgf(slap)
c moment generating function for a line
      include 'params.for'
      external initje
      double complex slap, sglmgf, sglshelf, expmgf
      external expmgf
      if (shelfswitch.eq.0) then
          binmgf = sglmgf(slap, expmgf)*expmgf(slap)
*      binmgf = sglmgf(slap, expmgf)                             
      else
          binmgf = sglshelf(slap, expmgf)*expmgf(slap)
      endif    
      return
      end
                  
      double complex function plymgf(slap)
c single electron spectral types      
      include 'params.for'
      external initje

      double complex slap
      
      plymgf = (one/(one + gain*slap/polyaH))**polyaH
      return
      end
      
      double complex function expmgf(slap)
c case when polyaH goes to infinity -- mgf for delta function ses
      include 'params.for'
      external initje
      
      double complex slap
      
      expmgf = exp(-gain*slap)
      return
      end
      
      double complex function dgmgf(slap)      
c case when gain varies between gain - dgain and gain + dgain
      include 'params.for'
      external initje

      double complex slap
      double complex temp, temp1, temp2
      
      if (slap.eq.(0.0,0.0)) then
          dgmgf = (1.0,0.0)
          return
      endif     
      if (dgain.gt.pebble) then
          if (abs(polyaH - one).gt.pebble) then
              temp1 = polyaH + (gain - dgain)*slap
              temp2 = polyaH + (gain + dgain)*slap
              temp = temp1/(temp1/polyaH)**polyaH - 
     1               temp2/(temp2/polyaH)**polyaH
              dgmgf = temp/(2.*dgain*slap*(polyaH-one))
              return
          else
              temp1 = one + (gain - dgain)*slap
              temp2 = one + (gain + dgain)*slap
              dgmgf = log(temp2/temp1)/(2.*dgain*slap)
              return
          endif        
      else
          dgmgf =  (one/(one + gain*slap/polyaH))**polyaH
          return
      endif
      end 

      double complex function sglmgf(slap, sesmgf) 
c mgf for a line without broadening and extra electron
      include 'params.for'
      external initje

      double complex slap, sesmgf
      double precision pFano, nPhot
      double complex temp 
c local variables
**      print *, 'slap in sglmgf = ', slap
      pFano = one - fano
      nPhot = (eChar - eOff)/ionEn
      temp = sesmgf(slap)
c channel shift and broadening
      if (abs(pFano) .gt. pebble) then
         sglmgf = (one + pFano*(temp - one))**(nPhot/pFano)
c general jm case
      else
           sglmgf = exp((temp - one)*nPhot)
c jm-Prescott intermediate case (Fano factor = 1)
      endif
      return
      end
            
      double complex function sglshelf(slap, sesmgf) 
c mgf for a line without broadening and extra electron
      include 'params.for'
      external initje

      double complex slap, sesmgf

      double precision pFano, nPhot
      double complex temp, temp1, temp6, temp7, temp0
      double precision nupper, temp2, temp3, temp4, temp5
      double complex shelfpart, tpart1, tpart2
c local variables
**      print *, 'slap in sglshelf = ', slap
      pFano = one - fano
      nPhot = (eChar - eOff)/ionEn
      temp = sesmgf(slap)

      if (temp.eq.(1.0,0.0)) then
          sglshelf = (1.0,0.0)
          return
      endif
c this point evaluated separately because of mild apparent 
c singularities
          
      if (abs(pFano) .gt. pebble) then
         temp1 = fano + pFano*temp
         temp7 = - log(temp1)
         nupper = nPhot/pFano
         temp6 = temp1**nupper
         temp0 =  (one + pFano*(temp - one))**nupper    
c the JM case for a line is temp0
         if (shelfnorm.ne.0.0) then 
            shelfpart = (pFano*(one - temp6))/(nPhot*temp7)
         else
            shelfpart = (0.0D0, 0.0D0)
         endif
         if (t1norm.ne.0.0) then 
              temp2 = t1par**nPhot
              temp3 = pFano*log(t1par)
              if (temp2.gt.one) then
                   temp2 = one/temp2
c taking reciprocal averts overflow problem 11/4/96
                   tpart1 = (temp6 - temp2)/
     1             ((one - temp2)*(one - temp7/temp3)) 
              else 
                  tpart1 = (temp2*temp6 - one)/
     1            ((temp2 - one)*(one - temp7/temp3))
              endif
         else
              tpart1 = (0.0D0, 0.0D0)
         endif
c
         if (t2norm.ne.0.0) then
              temp4 = t2par**nPhot
              temp5 = pFano*log(t2par)
              if (temp4.gt.one) then
                   temp4 = one/temp4
c taking reciprocal averts overflow problem 11/4/96
                   tpart2 = (temp6 - temp4)/
     1             ((one - temp4)*(one - temp7/temp5))
              else
                   tpart2 = (temp4*temp6 - one)/
     1             ((temp4 - one)*(one - temp7/temp5))
              endif
         else
              tpart2 = (0.0D0, 0.0D0)
         endif
c general jm case

      else
          temp0 = exp(nPhot*(temp - one))
c the Prescott case for the line is temp0 
          if (shelfnorm.ne.0.0) then
             shelfpart = (temp0 - one)/((temp - one)*nPhot)
          else
             shelfpart = (0.0D0, 0.0D0)
          endif
          if (t1norm.ne.0.) then
                temp2 = t1par**nPhot
                tpart1 = (- one + temp2*temp0)/
     1          ((temp2 - one)*(one + (temp - one)/log(t1par)))
          else
             tpart1 = (0.0D0, 0.0D0)
          endif
c
          if (t2norm.ne.0.) then
                temp4 = t2par**nPhot
                tpart2 = (- one + temp4*temp0)/
     1          ((temp4 - one)*(one + (temp - one)/log(t2par)))
          else
             tpart2 = (0.0D0, 0.0D0)
          endif
c jm-Prescott intermediate case (Fano factor = 1)                    
      endif

*      print *, "Within jmmgf1, fano, pFano, 
*     1   slap, temp, temp0, temp1, temp2, temp3, temp4, temp5, 
*     2   temp6, nPhot, tpart1, tpart2, shelfpart, t1norm, t2norm, 
*     3   shelfnorm = ", fano, pFano,  
*     4   slap, temp, temp0, temp1, temp2, temp3, temp4, temp5, 
*     5   temp6, nPhot, tpart1, tpart2, shelfpart, t1norm, t2norm, 
*     6   shelfnorm
c
c The following shift was applied to the falling tail to bring
c it to the high-energy side of the main line.

      tpart1 = tpart1 * exp(- nPhot * gain * slap)

         sglshelf = (temp0 +
     1            t1norm*tpart1 + t2norm*tpart2 + shelfnorm*shelfpart)/
     2            (one + t1norm + t2norm + shelfnorm)
c sglshelf includes the line, the shelf, and two tails
 
      return
      end
            
c
      double complex function plsrmg(slap)
c mgf for the pulser which is assumed to have a Gaussian shape       
      include 'params.for'
      external initje

      double complex slap
      
      if (slap.eq.(0.0,0.0)) then
          plsrmg = (1.0,0.0)
          return
      endif    
      
      if (shift.eq.0.0.and.sigma.eq.0.0) then
          plsrmg = one
          return
      elseif (shift.eq.0.0) then
          plsrmg = exp(half*(slap*sigma)**2)
          return
      elseif (sigma.eq.0.0) then
          plsrmg = exp(-slap*shift)
          return
      else        
          plsrmg =  exp(-slap*shift + half*(slap*sigma)**2)
      return
      endif
      end

      double complex function brshft(slap)
c mgf for shifting and broadening       
      include 'params.for'
      external initje

      double complex slap
      
      if (slap.eq.(0.0,0.0)) then
          brshft = (1.0,0.0)
          return
      endif    
      
      if (shift.eq.0.0.and.sigma.eq.0.0) then
          brshft = one
          return
      elseif (shift.eq.0.0) then
          brshft = exp(half*(slap*sigma)**2)
          return
      elseif (sigma.eq.0.0) then
          brshft = exp(-slap*shift)
          return
      else        
          brshft =  exp(-slap*shift + half*(slap*sigma)**2)
      return
      endif
      end

